@@ -1,32 +1,41 @@
-/***********************************************************************/
-/*                                                                     */
-/*                                OCaml                                */
-/*                                                                     */
-/*             Damien Doligez, projet Para, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License, with    */
-/*  the special exception on linking described in file ../LICENSE.     */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*              Damien Doligez, projet Para, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
+
+#define CAML_INTERNALS
 
 #include "caml/alloc.h"
+#include "caml/backtrace.h"
 #include "caml/compact.h"
 #include "caml/custom.h"
+#include "caml/fail.h"
 #include "caml/finalise.h"
 #include "caml/freelist.h"
 #include "caml/gc.h"
 #include "caml/gc_ctrl.h"
 #include "caml/major_gc.h"
+#include "caml/memory.h"
 #include "caml/minor_gc.h"
 #include "caml/misc.h"
 #include "caml/mlvalues.h"
+#include "caml/signals.h"
 #ifdef NATIVE_CODE
-#include "stack.h"
+#include "caml/stack.h"
 #else
 #include "caml/stacks.h"
 #endif
+#include "caml/startup_aux.h"
 
 #ifndef NATIVE_CODE
 extern uintnat caml_max_stack_size;    /* defined in stacks.c */
@@ -38,8 +47,8 @@ double caml_stat_minor_words = 0.0,
 
 intnat caml_stat_minor_collections = 0,
        caml_stat_major_collections = 0,
-       caml_stat_heap_size = 0,              /* bytes */
-       caml_stat_top_heap_size = 0,          /* bytes */
+       caml_stat_heap_wsz = 0,
+       caml_stat_top_heap_wsz = 0,
        caml_stat_compactions = 0,
        caml_stat_heap_chunks = 0;
 
@@ -48,7 +57,7 @@ extern uintnat caml_percent_free;         /*        see major_gc.c */
 extern uintnat caml_percent_max;          /*        see compact.c */
 extern uintnat caml_allocation_policy;    /*        see freelist.c */
 
-#define Next(hp) ((hp) + Bhsize_hp (hp))
+#define Next(hp) ((hp) + Whsize_hp (hp))
 
 #ifdef DEBUG
 
@@ -77,7 +86,7 @@ static void check_head (value v)
   }
 }
 
-static void check_block (char *hp)
+static void check_block (header_t *hp)
 {
   mlsize_t i;
   value v = Val_hp (hp);
@@ -127,9 +136,9 @@ static value heap_stats (int returnstats)
          free_words = 0, free_blocks = 0, largest_free = 0,
          fragments = 0, heap_chunks = 0;
   char *chunk = caml_heap_start, *chunk_end;
-  char *cur_hp;
+  header_t *cur_hp;
 #ifdef DEBUG
-  char *prev_hp;
+  header_t *prev_hp;
 #endif
   header_t cur_hd;
 
@@ -143,19 +152,20 @@ static value heap_stats (int returnstats)
 #ifdef DEBUG
     prev_hp = NULL;
 #endif
-    cur_hp = chunk;
-    while (cur_hp < chunk_end){
+    cur_hp = (header_t *) chunk;
+    while (cur_hp < (header_t *) chunk_end){
       cur_hd = Hd_hp (cur_hp);
-                                           Assert (Next (cur_hp) <= chunk_end);
+      Assert (Next (cur_hp) <= (header_t *) chunk_end);
       switch (Color_hd (cur_hd)){
       case Caml_white:
         if (Wosize_hd (cur_hd) == 0){
           ++ fragments;
           Assert (prev_hp == NULL
                   || Color_hp (prev_hp) != Caml_blue
-                  || cur_hp == caml_gc_sweep_hp);
+                  || cur_hp == (header_t *) caml_gc_sweep_hp);
         }else{
-          if (caml_gc_phase == Phase_sweep && cur_hp >= caml_gc_sweep_hp){
+          if (caml_gc_phase == Phase_sweep
+              && cur_hp >= (header_t *) caml_gc_sweep_hp){
             ++ free_blocks;
             free_words += Whsize_hd (cur_hd);
             if (Whsize_hd (cur_hd) > largest_free){
@@ -201,27 +211,30 @@ static value heap_stats (int returnstats)
       prev_hp = cur_hp;
 #endif
       cur_hp = Next (cur_hp);
-    }                                          Assert (cur_hp == chunk_end);
+    }                             Assert (cur_hp == (header_t *) chunk_end);
     chunk = Chunk_next (chunk);
   }
 
+#ifdef DEBUG
+  caml_final_invariant_check();
+#endif
+
   Assert (heap_chunks == caml_stat_heap_chunks);
-  Assert (live_words + free_words + fragments
-          == Wsize_bsize (caml_stat_heap_size));
+  Assert (live_words + free_words + fragments == caml_stat_heap_wsz);
 
   if (returnstats){
     CAMLlocal1 (res);
 
     /* get a copy of these before allocating anything... */
     double minwords = caml_stat_minor_words
-                      + (double) Wsize_bsize (caml_young_end - caml_young_ptr);
+                      + (double) (caml_young_alloc_end - caml_young_ptr);
     double prowords = caml_stat_promoted_words;
     double majwords = caml_stat_major_words + (double) caml_allocated_words;
     intnat mincoll = caml_stat_minor_collections;
     intnat majcoll = caml_stat_major_collections;
-    intnat heap_words = Wsize_bsize (caml_stat_heap_size);
+    intnat heap_words = caml_stat_heap_wsz;
     intnat cpct = caml_stat_compactions;
-    intnat top_heap_words = Wsize_bsize (caml_stat_top_heap_size);
+    intnat top_heap_words = caml_stat_top_heap_wsz;
 
     res = caml_alloc_tuple (16);
     Store_field (res, 0, caml_copy_double (minwords));
@@ -255,8 +268,12 @@ void caml_heap_check (void)
 
 CAMLprim value caml_gc_stat(value v)
 {
+  value result;
+  CAML_INSTR_SETUP (tmr, "");
   Assert (v == Val_unit);
-  return heap_stats (1);
+  result = heap_stats (1);
+  CAML_INSTR_TIME (tmr, "explicit/gc_stat");
+  return result;
 }
 
 CAMLprim value caml_gc_quick_stat(value v)
@@ -266,13 +283,13 @@ CAMLprim value caml_gc_quick_stat(value v)
 
   /* get a copy of these before allocating anything... */
   double minwords = caml_stat_minor_words
-                    + (double) Wsize_bsize (caml_young_end - caml_young_ptr);
+                    + (double) (caml_young_alloc_end - caml_young_ptr);
   double prowords = caml_stat_promoted_words;
   double majwords = caml_stat_major_words + (double) caml_allocated_words;
   intnat mincoll = caml_stat_minor_collections;
   intnat majcoll = caml_stat_major_collections;
-  intnat heap_words = caml_stat_heap_size / sizeof (value);
-  intnat top_heap_words = caml_stat_top_heap_size / sizeof (value);
+  intnat heap_words = caml_stat_heap_wsz;
+  intnat top_heap_words = caml_stat_top_heap_wsz;
   intnat cpct = caml_stat_compactions;
   intnat heap_chunks = caml_stat_heap_chunks;
 
@@ -296,6 +313,18 @@ CAMLprim value caml_gc_quick_stat(value v)
   CAMLreturn (res);
 }
 
+double caml_gc_minor_words_unboxed()
+{
+  return (caml_stat_minor_words
+          + (double) (caml_young_alloc_end - caml_young_ptr));
+}
+
+CAMLprim value caml_gc_minor_words(value v)
+{
+  CAMLparam0 ();   /* v is ignored */
+  CAMLreturn(caml_copy_double(caml_gc_minor_words_unboxed()));
+}
+
 CAMLprim value caml_gc_counters(value v)
 {
   CAMLparam0 ();   /* v is ignored */
@@ -303,7 +332,7 @@ CAMLprim value caml_gc_counters(value v)
 
   /* get a copy of these before allocating anything... */
   double minwords = caml_stat_minor_words
-                    + (double) Wsize_bsize (caml_young_end - caml_young_ptr);
+                    + (double) (caml_young_alloc_end - caml_young_ptr);
   double prowords = caml_stat_promoted_words;
   double majwords = caml_stat_major_words + (double) caml_allocated_words;
 
@@ -314,13 +343,18 @@ CAMLprim value caml_gc_counters(value v)
   CAMLreturn (res);
 }
 
+CAMLprim value caml_gc_huge_fallback_count (value v)
+{
+  return Val_long (caml_huge_fallback_count);
+}
+
 CAMLprim value caml_gc_get(value v)
 {
   CAMLparam0 ();   /* v is ignored */
   CAMLlocal1 (res);
 
-  res = caml_alloc_tuple (7);
-  Store_field (res, 0, Val_long (Wsize_bsize (caml_minor_heap_size)));  /* s */
+  res = caml_alloc_tuple (8);
+  Store_field (res, 0, Val_long (caml_minor_heap_wsz));                 /* s */
   Store_field (res, 1, Val_long (caml_major_heap_increment));           /* i */
   Store_field (res, 2, Val_long (caml_percent_free));                   /* o */
   Store_field (res, 3, Val_long (caml_verb_gc));                        /* v */
@@ -331,6 +365,7 @@ CAMLprim value caml_gc_get(value v)
   Store_field (res, 5, Val_long (0));
 #endif
   Store_field (res, 6, Val_long (caml_allocation_policy));              /* a */
+  Store_field (res, 7, Val_long (caml_major_window));                   /* w */
   CAMLreturn (res);
 }
 
@@ -353,12 +388,20 @@ static intnat norm_minsize (intnat s)
   return s;
 }
 
+static uintnat norm_window (intnat w)
+{
+  if (w < 1) w = 1;
+  if (w > Max_major_window) w = Max_major_window;
+  return w;
+}
+
 CAMLprim value caml_gc_set(value v)
 {
   uintnat newpf, newpm;
   asize_t newheapincr;
-  asize_t newminsize;
+  asize_t newminwsz;
   uintnat oldpolicy;
+  CAML_INSTR_SETUP (tmr, "");
 
   caml_verb_gc = Long_val (Field (v, 3));
 
@@ -396,20 +439,35 @@ CAMLprim value caml_gc_set(value v)
                      caml_allocation_policy);
   }
 
+  /* This field was added in 4.03.0. */
+  if (Wosize_val (v) >= 8){
+    int old_window = caml_major_window;
+    caml_set_major_window (norm_window (Long_val (Field (v, 7))));
+    if (old_window != caml_major_window){
+      caml_gc_message (0x20, "New smoothing window size: %d\n",
+                       caml_major_window);
+    }
+  }
+
     /* Minor heap size comes last because it will trigger a minor collection
        (thus invalidating [v]) and it can raise [Out_of_memory]. */
-  newminsize = Bsize_wsize (norm_minsize (Long_val (Field (v, 0))));
-  if (newminsize != caml_minor_heap_size){
-    caml_gc_message (0x20, "New minor heap size: %luk bytes\n",
-                     newminsize/1024);
-    caml_set_minor_heap_size (newminsize);
+  newminwsz = norm_minsize (Long_val (Field (v, 0)));
+  if (newminwsz != caml_minor_heap_wsz){
+    caml_gc_message (0x20, "New minor heap size: %luk words\n",
+                     newminwsz / 1024);
+    caml_set_minor_heap_size (Bsize_wsize (newminwsz));
   }
+  CAML_INSTR_TIME (tmr, "explicit/gc_set");
   return Val_unit;
 }
 
 CAMLprim value caml_gc_minor(value v)
-{                                                    Assert (v == Val_unit);
-  caml_minor_collection ();
+{
+  CAML_INSTR_SETUP (tmr, "");
+  Assert (v == Val_unit);
+  caml_request_minor_gc ();
+  caml_gc_dispatch ();
+  CAML_INSTR_TIME (tmr, "explicit/gc_minor");
   return Val_unit;
 }
 
@@ -417,30 +475,34 @@ static void test_and_compact (void)
 {
   float fp;
 
-  fp = 100.0 * caml_fl_cur_size
-       / (Wsize_bsize (caml_stat_heap_size) - caml_fl_cur_size);
+  fp = 100.0 * caml_fl_cur_wsz / (caml_stat_heap_wsz - caml_fl_cur_wsz);
   if (fp > 999999.0) fp = 999999.0;
   caml_gc_message (0x200, "Estimated overhead (lower bound) = %"
                           ARCH_INTNAT_PRINTF_FORMAT "u%%\n",
                    (uintnat) fp);
-  if (fp >= caml_percent_max && caml_stat_heap_chunks > 1){
+  if (fp >= caml_percent_max){
     caml_gc_message (0x200, "Automatic compaction triggered.\n", 0);
     caml_compact_heap ();
   }
 }
 
 CAMLprim value caml_gc_major(value v)
-{                                                    Assert (v == Val_unit);
+{
+  CAML_INSTR_SETUP (tmr, "");
+  Assert (v == Val_unit);
   caml_gc_message (0x1, "Major GC cycle requested\n", 0);
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
   test_and_compact ();
   caml_final_do_calls ();
+  CAML_INSTR_TIME (tmr, "explicit/gc_major");
   return Val_unit;
 }
 
 CAMLprim value caml_gc_full_major(value v)
-{                                                    Assert (v == Val_unit);
+{
+  CAML_INSTR_SETUP (tmr, "");
+  Assert (v == Val_unit);
   caml_gc_message (0x1, "Full major GC cycle requested\n", 0);
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
@@ -449,18 +511,23 @@ CAMLprim value caml_gc_full_major(value v)
   caml_finish_major_cycle ();
   test_and_compact ();
   caml_final_do_calls ();
+  CAML_INSTR_TIME (tmr, "explicit/gc_full_major");
   return Val_unit;
 }
 
 CAMLprim value caml_gc_major_slice (value v)
 {
+  CAML_INSTR_SETUP (tmr, "");
   Assert (Is_long (v));
-  caml_empty_minor_heap ();
-  return Val_long (caml_major_collection_slice (Long_val (v)));
+  caml_major_collection_slice (Long_val (v));
+  CAML_INSTR_TIME (tmr, "explicit/gc_major_slice");
+  return Val_long (0);
 }
 
 CAMLprim value caml_gc_compaction(value v)
-{                                                    Assert (v == Val_unit);
+{
+  CAML_INSTR_SETUP (tmr, "");
+  Assert (v == Val_unit);
   caml_gc_message (0x10, "Heap compaction requested\n", 0);
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
@@ -469,9 +536,35 @@ CAMLprim value caml_gc_compaction(value v)
   caml_finish_major_cycle ();
   caml_compact_heap ();
   caml_final_do_calls ();
+  CAML_INSTR_TIME (tmr, "explicit/gc_compact");
   return Val_unit;
 }
 
+CAMLprim value caml_get_minor_free (value v)
+{
+  return Val_int (caml_young_ptr - caml_young_alloc_start);
+}
+
+CAMLprim value caml_get_major_bucket (value v)
+{
+  long i = Long_val (v);
+  if (i < 0) caml_invalid_argument ("Gc.get_bucket");
+  if (i < caml_major_window){
+    i += caml_major_ring_index;
+    if (i >= caml_major_window) i -= caml_major_window;
+    CAMLassert (0 <= i && i < caml_major_window);
+    return Val_long ((long) (caml_major_ring[i] * 1e6));
+  }else{
+    return Val_long (0);
+  }
+}
+
+CAMLprim value caml_get_major_credit (value v)
+{
+  CAMLassert (v == Val_unit);
+  return Val_long ((long) (caml_major_work_credit * 1e6));
+}
+
 uintnat caml_normalize_heap_increment (uintnat i)
 {
   if (i < Bsize_wsize (Heap_chunk_min)){
@@ -480,13 +573,19 @@ uintnat caml_normalize_heap_increment (uintnat i)
   return ((i + Page_size - 1) >> Page_log) << Page_log;
 }
 
+/* [minor_size] and [major_size] are numbers of words
+   [major_incr] is either a percentage or a number of words */
 void caml_init_gc (uintnat minor_size, uintnat major_size,
                    uintnat major_incr, uintnat percent_fr,
-                   uintnat percent_m)
+                   uintnat percent_m, uintnat window)
 {
   uintnat major_heap_size =
     Bsize_wsize (caml_normalize_heap_increment (major_size));
 
+  CAML_INSTR_INIT ();
+  if (caml_init_alloc_for_heap () != 0){
+    caml_fatal_error ("cannot initialize heap: mmap failed\n");
+  }
   if (caml_page_table_initialize(Bsize_wsize(minor_size) + major_heap_size)){
     caml_fatal_error ("OCaml runtime error: cannot initialize page table\n");
   }
@@ -495,8 +594,9 @@ void caml_init_gc (uintnat minor_size, uintnat major_size,
   caml_percent_free = norm_pfree (percent_fr);
   caml_percent_max = norm_pmax (percent_m);
   caml_init_major_heap (major_heap_size);
-  caml_gc_message (0x20, "Initial minor heap size: %luk bytes\n",
-                   caml_minor_heap_size / 1024);
+  caml_major_window = norm_window (window);
+  caml_gc_message (0x20, "Initial minor heap size: %luk words\n",
+                   caml_minor_heap_wsz / 1024);
   caml_gc_message (0x20, "Initial major heap size: %luk bytes\n",
                    major_heap_size / 1024);
   caml_gc_message (0x20, "Initial space overhead: %lu%%\n", caml_percent_free);
@@ -510,4 +610,64 @@ void caml_init_gc (uintnat minor_size, uintnat major_size,
   }
   caml_gc_message (0x20, "Initial allocation policy: %d\n",
                    caml_allocation_policy);
+  caml_gc_message (0x20, "Initial smoothing window: %d\n",
+                   caml_major_window);
+}
+
+
+/* FIXME After the startup_aux.c unification, move these functions there. */
+
+CAMLprim value caml_runtime_variant (value unit)
+{
+  CAMLassert (unit == Val_unit);
+#if defined (DEBUG)
+  return caml_copy_string ("d");
+#elif defined (CAML_INSTR)
+  return caml_copy_string ("i");
+#else
+  return caml_copy_string ("");
+#endif
+}
+
+extern int caml_parser_trace;
+
+CAMLprim value caml_runtime_parameters (value unit)
+{
+  CAMLassert (unit == Val_unit);
+  return caml_alloc_sprintf
+    ("a=%d,b=%s,H=%lu,i=%lu,l=%lu,o=%lu,O=%lu,p=%d,s=%lu,t=%d,v=%lu,w=%d,W=%lu",
+     /* a */ caml_allocation_policy,
+     /* b */ caml_backtrace_active,
+     /* h */ /* missing */ /* FIXME add when changed to min_heap_size */
+     /* H */ caml_use_huge_pages,
+     /* i */ caml_major_heap_increment,
+#ifdef NATIVE_CODE
+     /* l */ 0,
+#else
+     /* l */ caml_max_stack_size,
+#endif
+     /* o */ caml_percent_free,
+     /* O */ caml_percent_max,
+     /* p */ caml_parser_trace,
+     /* R */ /* missing */
+     /* s */ caml_minor_heap_wsz,
+     /* t */ caml_trace_level,
+     /* v */ caml_verb_gc,
+     /* w */ caml_major_window,
+     /* W */ caml_runtime_warnings
+     );
+}
+
+/* Control runtime warnings */
+
+CAMLprim value caml_ml_enable_runtime_warnings(value vbool)
+{
+  caml_runtime_warnings = Bool_val(vbool);
+  return Val_unit;
+}
+
+CAMLprim value caml_ml_runtime_warnings_enabled(value unit)
+{
+  CAMLassert (unit == Val_unit);
+  return Val_bool(caml_runtime_warnings);
 }
