@@ -1,14 +1,17 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*            Maxence Guesdon, projet Cristal, INRIA Rocquencourt      *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Maxence Guesdon, projet Cristal, INRIA Rocquencourt        *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Analysis of implementation files. *)
 open Misc
@@ -21,7 +24,6 @@ let print_DEBUG s = print_string s ; print_newline ();;
 
 type typedtree = (Typedtree.structure * Typedtree.module_coercion)
 
-module Name = Odoc_name
 open Odoc_parameter
 open Odoc_value
 open Odoc_type
@@ -31,12 +33,6 @@ open Odoc_class
 open Odoc_module
 open Odoc_types
 
-(** This variable contains the regular expression representing a blank.*)
-let blank = "[ \010\013\009\012']"
-
-(** This variable contains the regular expression representing a blank but not a '\n'.*)
-let simple_blank = "[ \013\009\012]"
-
 (** This module is used to search for structure items by name in a Typedtree.structure.
    One function creates two hash tables, which can then be used to search for elements.
    Class elements do not use tables.
@@ -52,7 +48,6 @@ module Typedtree_search =
       | X of string
       | E of string
       | P of string
-      | IM of string
 
     type tab = (ele, Typedtree.structure_item_desc) Hashtbl.t
     type tab_values = (Odoc_module.Name.t, Typedtree.pattern * Typedtree.expression) Hashtbl.t
@@ -60,7 +55,7 @@ module Typedtree_search =
     let iter_val_pattern = function
       | Typedtree.Tpat_any -> None
       | Typedtree.Tpat_var (name, _) -> Some (Name.from_ident name)
-      | Typedtree.Tpat_tuple _ -> None (* A VOIR quand on traitera les tuples *)
+      | Typedtree.Tpat_tuple _ -> None (* FIXME when we will handle tuples *)
       | _ -> None
 
     let add_to_hashes table table_values tt =
@@ -83,21 +78,21 @@ module Typedtree_search =
         end
       | Typedtree.Tstr_exception ext ->
           Hashtbl.add table (E (Name.from_ident ext.ext_id)) tt
-      | Typedtree.Tstr_type ident_type_decl_list ->
+      | Typedtree.Tstr_type (rf, ident_type_decl_list) ->
           List.iter
             (fun td ->
               Hashtbl.add table (T (Name.from_ident td.typ_id))
-                (Typedtree.Tstr_type [td]))
+                (Typedtree.Tstr_type (rf, [td])))
             ident_type_decl_list
       | Typedtree.Tstr_class info_list ->
           List.iter
-            (fun (ci, m, s) ->
+            (fun (ci, s) ->
               Hashtbl.add table (C (Name.from_ident ci.ci_id_class))
-                (Typedtree.Tstr_class [ci, m, s]))
+                (Typedtree.Tstr_class [ci, s]))
             info_list
       | Typedtree.Tstr_class_type info_list ->
           List.iter
-            (fun ((id,id_loc,_) as ci) ->
+            (fun ((id,_,_) as ci) ->
               Hashtbl.add table
                 (CT (Name.from_ident id))
                 (Typedtree.Tstr_class_type [ci]))
@@ -145,12 +140,12 @@ module Typedtree_search =
 
     let search_type_declaration table name =
       match Hashtbl.find table (T name) with
-      | (Typedtree.Tstr_type [td]) -> td
+      | (Typedtree.Tstr_type (_, [td])) -> td
       | _ -> assert false
 
     let search_class_exp table name =
       match Hashtbl.find table (C name) with
-      | (Typedtree.Tstr_class [(ci, _, _ )]) ->
+      | (Typedtree.Tstr_class [(ci, _ )]) ->
           let ce = ci.ci_expr in
           (
            try
@@ -189,10 +184,10 @@ module Typedtree_search =
       let rec iter = function
         | [] ->
             raise Not_found
-        | { cf_desc = Typedtree.Tcf_val (_, _, ident, Tcfk_concrete (_, exp), _) } :: q
+        | { cf_desc = Typedtree.Tcf_val (_, _, ident, Tcfk_concrete (_, exp), _) } :: _
           when Name.from_ident ident = name ->
             exp.Typedtree.exp_type
-        | { cf_desc = Typedtree.Tcf_val (_, _, ident, Tcfk_virtual typ, _) } :: q
+        | { cf_desc = Typedtree.Tcf_val (_, _, ident, Tcfk_virtual typ, _) } :: _
           when Name.from_ident ident = name ->
             typ.Typedtree.ctyp_type
         | _ :: q ->
@@ -200,19 +195,11 @@ module Typedtree_search =
       in
       iter cls.Typedtree.cstr_fields
 
-    let class_sig_of_cltype_decl =
-      let rec iter = function
-        Types.Cty_constr (_, _, cty) -> iter cty
-      | Types.Cty_signature s -> s
-      | Types.Cty_arrow (_,_, cty) -> iter cty
-      in
-      fun ct_decl -> iter ct_decl.Types.clty_type
-
    let search_method_expression cls name =
       let rec iter = function
         | [] ->
             raise Not_found
-        | { cf_desc = Typedtree.Tcf_method (label, _, Tcfk_concrete (_, exp)) } :: q when label.txt = name ->
+        | { cf_desc = Typedtree.Tcf_method (label, _, Tcfk_concrete (_, exp)) } :: _ when label.txt = name ->
             exp
         | _ :: q ->
             iter q
@@ -301,9 +288,9 @@ module Analyser =
           (* This case means we have a 'function' without pattern, that's impossible *)
           raise (Failure "tt_analyse_function_parameters: 'function' without pattern")
 
-      | {c_lhs=pattern_param} :: second_ele :: q ->
+      | {c_lhs=pattern_param} :: _second_ele :: _ ->
           (* implicit pattern matching -> anonymous parameter and no more parameter *)
-          (* A VOIR : le label ? *)
+          (* FIXME : label ? *)
           let parameter = Odoc_parameter.Tuple ([], Odoc_env.subst_type env pattern_param.pat_type) in
           [ parameter ]
 
@@ -355,7 +342,7 @@ module Analyser =
      let tt_analyse_value env current_module_name comment_opt loc pat_exp rec_flag =
        let (pat, exp) = pat_exp in
        match (pat.pat_desc, exp.exp_desc) with
-         (Typedtree.Tpat_var (ident, _), Typedtree.Texp_function (_, pat_exp_list2, partial)) ->
+         (Typedtree.Tpat_var (ident, _), Typedtree.Texp_function (_, pat_exp_list2, _partial)) ->
            (* a new function is defined *)
            let name_pre = Name.from_ident ident in
            let name = Name.parens_if_infix name_pre in
@@ -404,13 +391,13 @@ module Analyser =
            in
            [ new_value ]
 
-       | (Typedtree.Tpat_tuple lpat, _) ->
+       | (Typedtree.Tpat_tuple _, _) ->
            (* new identifiers are defined *)
-           (* A VOIR : by now we don't accept to have global variables defined in tuples *)
+           (* FIXME : by now we don't accept to have global variables defined in tuples *)
            []
 
        | _ ->
-           (* something else, we don't care ? A VOIR *)
+           (* something else, we don't care ? FIXME *)
            []
 
     (** This function takes a Typedtree.class_expr and returns a string which can stand for the class name.
@@ -454,9 +441,9 @@ module Analyser =
            | l ->
                match l with
                  [] ->
-                   (* cas impossible, on l'a filtre avant *)
+                   (* impossible case, it has already been filtered *)
                    assert false
-               | {c_lhs=pattern_param} :: second_ele :: q ->
+               | {c_lhs=pattern_param} :: _second_ele :: _ ->
                    (* implicit pattern matching -> anonymous parameter *)
                    (* Note : We can't match this pattern if it is the first call to the function. *)
                    let new_param = Simple_name
@@ -514,7 +501,7 @@ module Analyser =
 
     (** Analysis of a [Parsetree.class_struture] and a [Typedtree.class_structure] to get a couple
        (inherited classes, class elements). *)
-    let analyse_class_structure env current_class_name tt_class_sig last_pos pos_limit p_cls tt_cls table =
+    let analyse_class_structure env current_class_name tt_class_sig last_pos pos_limit p_cls tt_cls _table =
       let rec iter acc_inher acc_fields last_pos = function
         | [] ->
             let s = get_string_of_file last_pos pos_limit in
@@ -699,14 +686,14 @@ module Analyser =
               Typedtree.Tcl_ident (p,_,_) -> Name.from_path p
             | _ ->
                 (* we try to get the name from the environment. *)
-                (* A VOIR : dommage qu'on n'ait pas un Tclass_ident :-( meme quand on a class tutu = toto *)
+                (* FIXME : Unfortunately, we don't have a Tclass_ident :-( even for a class tutu = toto *)
                 Name.from_longident lid.txt
           in
-          (* On n'a pas ici les parametres de type sous forme de Types.type_expr,
-             par contre on peut les trouver dans le class_type *)
+          (* Here, type parameters are not present as Types.type_expr,
+             however they can be found in the class_type *)
           let params =
             match tt_class_exp.Typedtree.cl_type with
-              Types.Cty_constr (p2, type_exp_list, cltyp) ->
+              Types.Cty_constr (_p2, type_exp_list, _cltyp) ->
                 (* cltyp is the class type for [type_exp_list] p *)
                 type_exp_list
             | _ ->
@@ -740,8 +727,8 @@ module Analyser =
           ([],
            Class_structure (inherited_classes, class_elements) )
 
-      | (Parsetree.Pcl_fun (label, expression_opt, pattern, p_class_expr2),
-         Typedtree.Tcl_fun (_, pat, ident_exp_list, tt_class_expr2, partial)) ->
+      | (Parsetree.Pcl_fun (_label, _expression_opt, _pattern, p_class_expr2),
+         Typedtree.Tcl_fun (_, pat, _ident_exp_list, tt_class_expr2, _partial)) ->
            (* we check that this is not an optional parameter with
               a default value. In this case, we look for the good parameter pattern *)
            let (parameter, next_tt_class_exp) =
@@ -786,9 +773,9 @@ module Analyser =
             (* we want an ident, or else the class applied will appear in the form object ... end,
                because if the class applied has no name, the code is kinda ugly, isn't it ? *)
             match tt_class_expr2.Typedtree.cl_desc with
-              Typedtree.Tcl_ident (p,_,_) -> Name.from_path p (* A VOIR : obtenir le nom complet *)
+              Typedtree.Tcl_ident (p,_,_) -> Name.from_path p (* FIXME : obtain the full name *)
             | _ ->
-                (* A VOIR : dommage qu'on n'ait pas un Tclass_ident :-( meme quand on a class tutu = toto *)
+                (* FIXME : Unfortunately, we don't have a Tclass_ident :-( even for a class tutu = toto *)
                 match p_class_expr2.Parsetree.pcl_desc with
                   Parsetree.Pcl_constr (lid, _) ->
                     (* we try to get the name from the environment. *)
@@ -797,7 +784,7 @@ module Analyser =
                     Odoc_messages.object_end
           in
           let param_exps = List.fold_left
-              (fun acc -> fun (_, exp_opt, _) ->
+              (fun acc -> fun (_, exp_opt) ->
                 match exp_opt with
                   None -> acc
                 | Some e -> acc @ [e])
@@ -826,13 +813,13 @@ module Analyser =
               env current_class_name comment_opt last_pos p_class_expr2
               tt_class_expr2 table
 
-      | (Parsetree.Pcl_constraint (p_class_expr2, p_class_type2),
+      | (Parsetree.Pcl_constraint (p_class_expr2, _p_class_type2),
          Typedtree.Tcl_constraint (tt_class_expr2, _, _, _, _)) ->
           let (l, class_kind) = analyse_class_kind
               env current_class_name comment_opt last_pos p_class_expr2
                 tt_class_expr2 table
             in
-            (* A VOIR : analyse du class type ? on n'a pas toutes les infos. cf. Odoc_sig.analyse_class_type_kind *)
+            (* FIXME analysis of the class type ? We don't have all the infos. cf. Odoc_sig.analyse_class_type_kind *)
           let class_type_kind =
             (*Sig.analyse_class_type_kind
               env
@@ -867,19 +854,16 @@ module Analyser =
           tt_class_exp
           table
       in
-      let cl =
-        {
-          cl_name = complete_name ;
-          cl_info = comment_opt ;
-          cl_type = cltype ;
-          cl_virtual = virt ;
-          cl_type_parameters = type_parameters ;
-          cl_kind = kind ;
-          cl_parameters = parameters ;
-          cl_loc = { loc_impl = Some loc ; loc_inter = None } ;
-        }
-      in
-      cl
+      {
+        cl_name = complete_name ;
+        cl_info = comment_opt ;
+        cl_type = cltype ;
+        cl_virtual = virt ;
+        cl_type_parameters = type_parameters ;
+        cl_kind = kind ;
+        cl_parameters = parameters ;
+        cl_loc = { loc_impl = Some loc ; loc_inter = None } ;
+      }
 
     (** Get a name from a module expression, or "struct ... end" if the module expression
        is not an ident of a constraint on an ident. *)
@@ -899,7 +883,7 @@ module Analyser =
         match item.str_desc with
           Typedtree.Tstr_include incl ->
             acc @ [
-                  { (* A VOIR : chercher dans les modules et les module types, avec quel env ? *)
+                  { (* FIXME : search within modules and module types, with which env ? *)
                     im_name = tt_name_from_module_expr incl.incl_mod ;
                     im_module = None ;
                     im_info = None ;
@@ -981,7 +965,7 @@ module Analyser =
         | Element_type t ->
              (function
                 Types.Sig_type (ident,_,_) ->
-                  (* A VOIR: il est possible que le detail du type soit cache *)
+                  (* FIXME: type details can be hidden *)
                   let n1 = Name.simple t.ty_name
                   and n2 = Ident.name ident in
                   n1 = n2
@@ -1043,18 +1027,15 @@ module Analyser =
           [] ->
             let s = get_string_of_file last_pos pos_limit in
             let (_, ele_coms) = My_ir.all_special !file_name s in
-            let ele_comments =
-              List.fold_left
-                (fun acc -> fun sc ->
-                  match sc.Odoc_types.i_desc with
-                    None ->
-                      acc
-                  | Some t ->
-                      acc @ [Element_module_comment t])
-                []
-                ele_coms
-            in
-            ele_comments
+            List.fold_left
+              (fun acc -> fun sc ->
+                 match sc.Odoc_types.i_desc with
+                   None ->
+                     acc
+                 | Some t ->
+                     acc @ [Element_module_comment t])
+              []
+              ele_coms
         | item :: q ->
             let (comment_opt, ele_comments) =
               get_comments_in_module last_pos item.Parsetree.pstr_loc.Location.loc_start.Lexing.pos_cnum
@@ -1080,7 +1061,7 @@ module Analyser =
       iter env last_pos parsetree
 
    (** Analysis of a parse tree structure item to obtain a new environment and a list of elements.*)
-   and analyse_structure_item env current_module_name loc pos_limit comment_opt parsetree_item_desc typedtree
+   and analyse_structure_item env current_module_name loc pos_limit comment_opt parsetree_item_desc _typedtree
         table table_values =
       print_DEBUG "Odoc_ast:analyse_struture_item";
       match parsetree_item_desc with
@@ -1098,7 +1079,7 @@ module Analyser =
           let rec iter_pat = function
             | Parsetree.Ppat_any -> None
             | Parsetree.Ppat_var name -> Some name
-            | Parsetree.Ppat_tuple _ -> None (* A VOIR quand on traitera les tuples *)
+            | Parsetree.Ppat_tuple _ -> None (* FIXME when we will handle tuples *)
             | Parsetree.Ppat_constraint (pat, _) -> iter_pat pat.Parsetree.ppat_desc
             | _ -> None
           in
@@ -1179,7 +1160,7 @@ module Analyser =
             let new_env = Odoc_env.add_value env new_value.val_name in
             (0, new_env, [Element_value new_value])
 
-      | Parsetree.Pstr_type name_typedecl_list ->
+      | Parsetree.Pstr_type (rf, name_typedecl_list) ->
           (* of (string * type_declaration) list *)
           let extended_env =
             List.fold_left
@@ -1191,14 +1172,9 @@ module Analyser =
               name_typedecl_list
           in
           let env =
-            let is_nonrec =
-              List.exists
-                (fun td ->
-                   List.exists (fun (n, _) -> n.txt = "nonrec")
-                     td.Parsetree.ptype_attributes)
-                name_typedecl_list
-            in
-            if is_nonrec then env else extended_env
+            match rf with
+            | Recursive -> extended_env
+            | Nonrecursive -> env
           in
           let rec f ?(first=false) maybe_more_acc last_pos name_type_decl_list =
             match name_type_decl_list with
@@ -1325,10 +1301,11 @@ module Analyser =
                     let new_xt =
                       match tt_ext.ext_kind with
                           Text_decl(args, ret_type) ->
+                          let xt_args =
+                            Sig.get_cstr_args new_env ext_loc_end args in
                             {
                               xt_name = complete_name;
-                              xt_args =
-                                List.map (fun ctyp -> Odoc_env.subst_type new_env ctyp.ctyp_type) args;
+                              xt_args;
                               xt_ret =
                                 may_map (fun ctyp -> Odoc_env.subst_type new_env ctyp.ctyp_type) ret_type;
                               xt_type_extension = new_te;
@@ -1339,7 +1316,7 @@ module Analyser =
                         | Text_rebind(path, _) ->
                             {
                               xt_name = complete_name;
-                              xt_args = [];
+                              xt_args = Cstr_tuple [];
                               xt_ret = None;
                               xt_type_extension = new_te;
                               xt_alias =
@@ -1382,13 +1359,12 @@ module Analyser =
               Text_decl(tt_args, tt_ret_type) ->
                 let loc_start = loc.Location.loc_start.Lexing.pos_cnum in
                 let loc_end =  loc.Location.loc_end.Lexing.pos_cnum in
+                let ex_args =
+                  Sig.get_cstr_args env loc_end tt_args in
                 {
                   ex_name = complete_name ;
                   ex_info = comment_opt ;
-                  ex_args =
-                    List.map
-                      (fun ctyp -> Odoc_env.subst_type new_env ctyp.ctyp_type)
-                      tt_args;
+                  ex_args;
                   ex_ret =
                     Misc.may_map
                       (fun ctyp -> Odoc_env.subst_type new_env ctyp.ctyp_type)
@@ -1407,7 +1383,7 @@ module Analyser =
                 {
                   ex_name = complete_name ;
                   ex_info = comment_opt ;
-                  ex_args = [] ;
+                  ex_args = Cstr_tuple [] ;
                   ex_ret = None ;
                   ex_alias =
                     Some { ea_name =
@@ -1448,7 +1424,7 @@ module Analyser =
              let new_env = Odoc_env.add_module env new_module.m_name in
              let new_env2 =
                match new_module.m_type with
-                 (* A VOIR : cela peut-il etre Tmty_ident ? dans ce cas, on aurait pas la signature *)
+                 (* FIXME : can this be Tmty_ident? In this case, we would'nt have the signature *)
                  Types.Mty_signature s ->
                    Odoc_env.add_signature new_env new_module.m_name
                      ~rel: (Name.simple new_module.m_name) s
@@ -1463,8 +1439,8 @@ module Analyser =
           )
 
       | Parsetree.Pstr_recmodule mods ->
-          (* A VOIR ICI pb: pas de lien avec les module type
-             dans les contraintes sur les modules *)
+          (* FIXME Here problem: no link with module types
+             in module constraints *)
           let new_env =
             List.fold_left
               (fun acc_env {Parsetree.pmb_name=name;pmb_expr=mod_exp} ->
@@ -1552,7 +1528,7 @@ module Analyser =
           let new_env = Odoc_env.add_module_type env mt.mt_name in
           let new_env2 =
             match sig_mtype with
-              (* A VOIR : cela peut-il etre Tmty_ident ? dans ce cas, on n'aurait pas la signature *)
+              (* FIXME : can this be Tmty_ident? In this case, we would'nt have the signature *)
               Some (Types.Mty_signature s) ->
                 Odoc_env.add_signature new_env mt.mt_name ~rel: (Name.simple mt.mt_name) s
             | _ ->
@@ -1561,7 +1537,7 @@ module Analyser =
           (0, new_env2, [ Element_module_type mt ])
 
       | Parsetree.Pstr_open _ ->
-          (* A VOIR : enrichir l'environnement quand open ? *)
+          (* FIXME : extend the environment after open? *)
           let ele_comments = match comment_opt with
             None -> []
           | Some i ->
@@ -1670,7 +1646,7 @@ module Analyser =
           in
           (0, new_env, f ~first: true loc.Location.loc_start.Lexing.pos_cnum class_type_decl_list)
 
-      | Parsetree.Pstr_include incl ->
+      | Parsetree.Pstr_include _ ->
           (* we add a dummy included module which will be replaced by a correct
              one at the end of the module analysis,
              to use the Path.t of the included modules in the typdtree. *)
@@ -1681,7 +1657,7 @@ module Analyser =
               im_info = comment_opt ;
             }
           in
-          (0, env, [ Element_included_module im ]) (* A VOIR : etendre l'environnement ? avec quoi ? *)
+          (0, env, [ Element_included_module im ]) (* FIXME: extend the environment? With what? *)
 
      (** Analysis of a [Parsetree.module_expr] and a name to return a [t_module].*)
      and analyse_module env current_module_name module_name comment_opt p_module_expr tt_module_expr =
@@ -1690,7 +1666,7 @@ module Analyser =
       let pos_start = loc.Location.loc_start.Lexing.pos_cnum in
       let pos_end = loc.Location.loc_end.Lexing.pos_cnum in
       let modtype =
-        (* A VOIR : Odoc_env.subst_module_type env  ? *)
+        (* FIXME : Odoc_env.subst_module_type env  ? *)
         tt_module_expr.Typedtree.mod_type
       in
       let m_code_intf =
@@ -1718,8 +1694,8 @@ module Analyser =
       }
       in
       match (p_module_expr.Parsetree.pmod_desc, tt_module_expr.Typedtree.mod_desc) with
-        (Parsetree.Pmod_ident longident, Typedtree.Tmod_ident (path, _))
-        | (Parsetree.Pmod_ident longident,
+        (Parsetree.Pmod_ident _, Typedtree.Tmod_ident (path, _))
+        | (Parsetree.Pmod_ident _,
            Typedtree.Tmod_constraint
              ({Typedtree.mod_desc = Typedtree.Tmod_ident (path, _)}, _, _, _))
           ->
@@ -1760,7 +1736,7 @@ module Analyser =
              }
            in
            let dummy_complete_name = (*Name.concat "__"*) param.mp_name in
-           (* TODO: A VOIR CE __ *)
+           (* TODO: FIX THIS  __ *)
            let new_env = Odoc_env.add_module env dummy_complete_name in
            let m_base2 = analyse_module
                new_env
@@ -1840,7 +1816,7 @@ module Analyser =
           }
 
       | (Parsetree.Pmod_unpack p_exp,
-         Typedtree.Tmod_unpack (t_exp, tt_modtype)) ->
+         Typedtree.Tmod_unpack (_t_exp, tt_modtype)) ->
           print_DEBUG ("Odoc_ast: case Parsetree.Pmod_unpack + Typedtree.Tmod_unpack "^module_name);
           let code =
             let loc = p_module_expr.Parsetree.pmod_loc in
@@ -1863,9 +1839,9 @@ module Analyser =
             m_kind = Module_unpack (code, alias) ;
           }
 
-      | (parsetree, typedtree) ->
+      | (_parsetree, _typedtree) ->
           (*DEBUG*)let s_parse =
-          (*DEBUG*)  match parsetree with
+          (*DEBUG*)  match _parsetree with
           (*DEBUG*)    Parsetree.Pmod_ident _ -> "Pmod_ident"
           (*DEBUG*)  | Parsetree.Pmod_structure _ -> "Pmod_structure"
           (*DEBUG*)  | Parsetree.Pmod_functor _ -> "Pmod_functor"
@@ -1875,7 +1851,7 @@ module Analyser =
           (*DEBUG*)  | Parsetree.Pmod_extension _ -> "Pmod_extension"
           (*DEBUG*)in
           (*DEBUG*)let s_typed =
-          (*DEBUG*)  match typedtree with
+          (*DEBUG*)  match _typedtree with
           (*DEBUG*)    Typedtree.Tmod_ident _ -> "Tmod_ident"
           (*DEBUG*)  | Typedtree.Tmod_structure _ -> "Tmod_structure"
           (*DEBUG*)  | Typedtree.Tmod_functor _ -> "Tmod_functor"
@@ -1907,7 +1883,7 @@ module Analyser =
        in
        prepare_file complete_source_file input_file;
        (* We create the t_module for this file. *)
-       let mod_name = String.capitalize (Filename.basename (Filename.chop_extension source_file)) in
+       let mod_name = String.capitalize_ascii (Filename.basename (Filename.chop_extension source_file)) in
        let (len,info_opt) = My_ir.first_special !file_name !file in
 
        (* we must complete the included modules *)
